% This program replicates the computations in Carroll's model using my own
% program in order to time it accordingly. It uses loops everywhere so that
% the timing goes in agreement with the model we have.

clear all;
format long;
tic
beta=0.96;   % time preference
rho=2;       % coefficient of relative risk aversion
n=100;        % number of grid points
G=1.01;      % permanent income growth rate
dep=0.90;    % depreciation rate
curlyepsilon=0.36;  % Cobb-Douglas production function parameter
permval=[0.90,1.00,1.10];   % permanent shock values
permprob=[0.25,0.50,0.25];  % permanent shock probabilities

kss=((((G^rho)/(beta*dep))-1)/curlyepsilon)^(1/(curlyepsilon-1));
ass=kss*G/dep;
%Grid on assets tomorrow
alphagrid=exp(linspace(0.00,log(3*ass),n))-1; % set up grid points

%Initial guess for M and C
C1=0:n-1;
C1=0.1*C1;
M1=0:n-1;

conv=100;
iter=0;
while conv>1e-6    
    iter=iter+1;
    for i=1:length(alphagrid);
    EUP=0;
        for j=1:length(permval)
            k=alphagrid(i).*dep./(G.*permval(j));
            R=1+curlyepsilon*k.^(curlyepsilon-1);
            W=(1-curlyepsilon)*(k.^curlyepsilon);
            mtomorrow=R.*k+W;
            if i==1
            mtomorrow=0; %To deal with no resources no consumption.
            end
            Cnextp=interp1(M1,C1,mtomorrow,'linear','extrap');
            EUP=EUP+dep*beta*R.*(G*permval(j)*Cnextp).^-rho*permprob(j);            
        end;
        C2(i)=(EUP).^(-1/rho);
    end
    M2=alphagrid+C2;
    conv=max(abs(C2-C1));
    C1=C2;
    M1=M2;
end
toc

figure;
plot(M1,C1)
xlabel('m')
ylabel('c')
title('Consumption Function for Macro Model')
axis([0 5 0 2])

%This part of the program computes the Euler Equation errors.

EUP=0
for j=1:length(permval)
        k=(M1-C1)*dep/(G*permval(j));
        R=1+curlyepsilon*k.^(curlyepsilon-1);
        W=(1-curlyepsilon)*(k.^curlyepsilon);
        mtomorrow=R.*k+W;
        mtomorrow(1)=0; %To deal with no resources no consumption.
        Cnextp=interp1(M1,C1,mtomorrow,'linear','extrap');
        EUP=EUP+dep*beta*R.*(G*permval(j)*Cnextp).^-rho*permprob(j);
end

EEerror=1-(EUP.^(-1/rho))./C1;
EEerrormax=max(abs(EEerror))
EEerrorlog=log10(abs(EEerror));
EEerrormaxlog=max(EEerrorlog)